/*_############################################################################
  _## 
  _##  SNMP4J-AgentJMX - JMXTableSupport.java  
  _## 
  _##  Copyright (C) 2006-2009  Frank Fock (SNMP4J.org)
  _##  
  _##  This program is free software; you can redistribute it and/or modify
  _##  it under the terms of the GNU General Public License version 2 as 
  _##  published by the Free Software Foundation.
  _##
  _##  This program is distributed in the hope that it will be useful,
  _##  but WITHOUT ANY WARRANTY; without even the implied warranty of
  _##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  _##  GNU General Public License for more details.
  _##
  _##  You should have received a copy of the GNU General Public License
  _##  along with this program; if not, write to the Free Software
  _##  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
  _##  MA  02110-1301  USA
  _##  
  _##########################################################################*/

package org.snmp4j.agent.mo.jmx;

import org.snmp4j.smi.OID;
import org.snmp4j.agent.mo.MOTableRow;
import java.util.Iterator;

/**
 * The <code>JMXTableSupport</code> defines the necessary interface to
 * map tabular JMX data to a SNMP4J-Agent table model. The interface
 * takes care of mapping JMX row identifiers to SNMP table indexes and vice
 * versa.
 *
 * @author Frank Fock
 * @version 1.0
 */
public interface JMXTableSupport {

  /**
   * Maps a native MBean row identifier object or index to a SNMP table index
   * OID.
   * @param tableOID
   *    the OID of the table ManagedObject (including the .1 of the entry
   *    object) for which indexes are mapped.
   * @param nativeRowID
   *    the native row ID object which identifies the row. If <code>null</code>
   *    then <code>nativeIndex</code> identifies the row.
   * @param nativeIndex
   *    the native enumerating index if rows are not identified by an object.
   * @return
   *    the SNMP row index OID (suffix).
   */
  OID mapToIndex(OID tableOID, Object nativeRowID, int nativeIndex);

  /**
   * Maps a SNMP row index to a native row identifier object.
   * @param tableOID
   *    the OID of the table ManagedObject (including the .1 of the entry
   *    object) for which indexes are mapped.
   * @param rowIndex
   *    a SNMP row index of the table specified by <code>tableOID</code>.
   * @return
   *    a native row identifier.
   */
  Object mapToRowId(OID tableOID, OID rowIndex);

  /**
   * Returns an Iterator over the row identifiers of the table.
   * @param tableOID
   *    the OID of the table ManagedObject (including the .1 of the entry
   *    object) for which indexes are mapped.
   * @return
   *   an Iterator of row identifiers.
   */
  Iterator rowIdIterator(OID tableOID);

  /**
   * Returns a tail iterator over the row identifiers of the table.
   * @param tableOID
   *    the OID of the table ManagedObject (including the .1 of the entry
   *    object) for which indexes are mapped.
   * @param firstRowId
   *    the lower bound (including) of the row identifiers to return.
   * @return
   *   an Iterator of row identifiers.
   */
  Iterator rowIdTailIterator(OID tableOID, Object firstRowId);

  /**
   * Gets the last SNMP index currently supported by the specified table.
   * @param tableOID
   *    the OID of the table ManagedObject (including the .1 of the entry
   *    object) for which indexes are mapped.
   * @return
   *    the last SNMP row index of the specified table.
   */
  OID getLastIndex(OID tableOID);

  /**
   * Returns the number of rows of the specified table.
   * @param tableOID
   *    the OID of the table ManagedObject (including the .1 of the entry
   *    object) for which indexes are mapped.
   * @return
   *    the number of rows in the table.
   */
  int getRowCount(OID tableOID);

  /**
   * Gets the row values for the specified row.
   * @param tableOID
   *    the OID of the table ManagedObject (including the .1 of the entry
   *    object) for which indexes are mapped.
   * @param row
   *    a <code>MOTableRow</code> instance which will be modified to hold the
   *    SNMP values of the row specified by <code>row</code>'s index value.
   * @return
   *    a SNMP error status that indicates whether to operation succeeded or
   *    not.
   */
  int getRow(OID tableOID, MOTableRow row);

  /**
   * Sets the value of a column (cell) of a row from the corresponding SNMP
   * conceptual row.
   * @param tableOID
   *    the OID of the table ManagedObject (including the .1 of the entry
   *    object) for which indexes are mapped.
   * @param row
   *    a <code>MOTableRow</code> instance that identifies the target row by its
   *    row index (indirectly) and that contains the SNMP values that replace
   *    the corresponding native values.
   * @param column
   *    the zero-based column index to set.
   * @return
   *    a SNMP error status that indicates whether to operation succeeded or
   *    not.
   */
  int setRow(OID tableOID, MOTableRow row, int column);

}
